@@ -448,7 +448,6 @@ mail_config_ews_ooo_page_constructed (GObject *object)
 	text = _("The messages specified below will be automatically sent to "
 		 "each internal and external person who sends a mail to you.");
 	widget = gtk_label_new (text);
-	gtk_label_set_max_width_chars (GTK_LABEL (widget), 80);
 	gtk_label_set_line_wrap (GTK_LABEL (widget), TRUE);
 	gtk_misc_set_alignment (GTK_MISC (widget), 0.0, 0.5);
 	gtk_box_pack_start (GTK_BOX (container), widget, FALSE, FALSE, 0);
@@ -745,9 +744,6 @@ mail_config_ews_ooo_page_submit (EMailConfigPage *page,
 	e_ews_oof_settings_set_internal_reply (priv->oof_settings, text);
 	g_free (text);
 
-	e_ews_oof_settings_set_external_audience (priv->oof_settings,
-		gtk_combo_box_get_active (GTK_COMBO_BOX (priv->external_audience)));
-
 	text_buffer = priv->external_reply;
 	gtk_text_buffer_get_bounds (text_buffer, &start, &end);
 	text = gtk_text_buffer_get_text (text_buffer, &start, &end, FALSE);
